<?php

namespace Andy\UploadManager\XF\Http;

use function count, in_array, is_array;

class Upload extends XFCP_Upload
{
	public function transformImage()
	{
		if ($this->transformed)
		{
			return $this;
		}
		$this->transformed = true;

		if (!$this->isImage || $this->imageContentUnsafe)
		{
			// do nothing, just let it error
			return $this;
		}
		
		//########################################
		// start add-on code
		//########################################
		
		// get options
		$options = \XF::options();
		
		// get options from Admin CP -> Setup -> Options -> Upload manager -> Add border
		$addBorder = $options->uploadManagerAddBorder;
		
		// get options from Admin CP -> Setup -> Options -> Upload manager -> Border width
		$borderWidth = $options->uploadManagerBorderWidth;
		
		// get options from Admin CP -> Setup -> Options -> Upload manager -> Border color GD
		$borderColorGd = $options->uploadManagerBorderColorGd;
        
		// get options from Admin CP -> Setup -> Options -> Upload manager -> Border color Imagick
		$borderColorImagick = $options->uploadManagerBorderColorImagick;
		
		// get options from Admin CP -> Options -> Attachments -> Default image processor
		$imageLibrary = $options->imageLibrary;
		
		// check condition
		if ($this->extension == 'png')
		{
			// get image details
			$imageDetails = @getimagesize($this->tempFile);

			// get variables
			$width = $imageDetails[0];
			$height = $imageDetails[1];
			
			// get resizeWidth
			$resizeWidth = $this->imageWidth * .5;
			$resizeWidth = floor($resizeWidth);
			
			// check condition
			if ($imageLibrary == 'imPecl')
			{
				$imagickTemp = new \Imagick();
				$imagickTemp->readImage($this->tempFile);
				$imagick = new \Imagick();               
				$imagick->newImage($width, $height, "white"); 
				$imagick->compositeimage($imagickTemp, \Imagick::COMPOSITE_OVER, 0, 0);
				$imagick->setImageFormat('jpg');
				
				$imageResolution = $imagickTemp->getImageResolution();

				if ($imageResolution['x'] == 56.69)
				{
					$imagick->setImageResolution(72,72);
					$imagick->resizeImage($resizeWidth, 0, \Imagick::FILTER_LANCZOS, 1);
				}
				
				$imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
				$imagick->setImageCompressionQuality(85);
				
				$imagick->writeImage($this->tempFile);
			}
			
			// check condition
			if ($imageLibrary != 'imPecl')
			{
				$inputFile = imagecreatefrompng($this->tempFile);
				
				$imageResolution = imageresolution($inputFile);
				
				if ($imageResolution[0] == 144)
				{
					$newWidth = $width * .5;
					$newWidth = floor($newWidth);
					
					$newHeight = $height * .5;
					$newHeight = floor($newHeight);
					
					$output = imagecreatetruecolor($newWidth, $newHeight);
					$white = imagecolorallocate($output, 255, 255, 255);
					imagefilledrectangle($output, 0, 0, $newWidth, $newHeight, $white);
					imagecopyresampled($output, $inputFile, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
				}
				
				if ($imageResolution[0] != 144)
				{
					$output = imagecreatetruecolor($width, $height);
					$white = imagecolorallocate($output, 255, 255, 255);
					imagefilledrectangle($output, 0, 0, $width, $height, $white);
					imagecopy($output, $inputFile, 0, 0, 0, 0, $width, $height);
				}
				
				imageresolution($output, 72);
				
				imagejpeg($output, $this->tempFile);
			}
			
			// update fileName
			$this->fileName = str_ireplace('.png', '.jpg', $this->fileName);
			
			//#################################################
			// add border
			//#################################################

			// check condition
			if ($addBorder)
			{
				// check condition
				if ($imageLibrary == 'imPecl')
				{      
					$imagick = new \Imagick();
					$imagick->readImage($this->tempFile);
					$imagick->shaveImage($borderWidth, $borderWidth);
					$imagick->borderImage($borderColorImagick, $borderWidth, $borderWidth);
					$imagick->writeImage($this->tempFile);
				}

				// check condition
				if ($imageLibrary != 'imPecl')
				{
					$newBorderWidth = $borderWidth * 2;
					$newWidth = $width - $newBorderWidth;
					$newHeight = $height - $newBorderWidth;
					$borderColorGdArray = explode(',', $borderColorGd);

					$inputFile = imagecreatefromjpeg($this->tempFile);
					$inputFile = imagecrop($inputFile, ['x' => $borderWidth, 'y' => $borderWidth, 'width' => $newWidth, 'height' => $newHeight]);

					$output = imagecreatetruecolor($width, $height);
					$newBorderGdColor = imagecolorallocate($output, $borderColorGdArray[0], $borderColorGdArray[1], $borderColorGdArray[2]);
					imagefilledrectangle($output, 0, 0, $width, $height, $newBorderGdColor);
					imagecopy($output, $inputFile, $borderWidth, $borderWidth, 0, 0, $newWidth, $newHeight);
					imagejpeg($output, $this->tempFile); 
				}
			}
		}
		
		//########################################
		// end add-on code
		//########################################

		$orientation = 0;
		if ($this->exif && !empty($this->exif['IFD0']['Orientation']) && $this->exif['IFD0']['Orientation'] > 1)
		{
			$orientation = $this->exif['IFD0']['Orientation'];
		}
		$transformRequired = ($orientation > 1);

		$maxWidth = $this->maxWidth;
		$maxHeight = $this->maxHeight;

		if ($orientation >= 5 && $orientation <= 8)
		{
			// after rotation the X and Y coords will be reversed,
			// so flip the limits to reflect the "after" value
			$maxHeight = $this->maxWidth;
			$maxWidth = $this->maxHeight;
		}

		$resizeRequired = (
			($maxWidth && $this->imageWidth > $maxWidth)
			|| ($maxHeight && $this->imageHeight > $maxHeight)
		);

		$optimizeRequired = $this->optimize && !$this->isOptimized;

		if ($resizeRequired || $transformRequired || $optimizeRequired)
		{
			$imageManager = \XF::app()->imageManager();
			if ($imageManager->canResize($this->imageWidth, $this->imageHeight))
			{
				$image = $imageManager->imageFromFile($this->tempFile);
				if ($image)
				{
					if ($resizeRequired)
					{
						$image->resize($maxWidth ?: $maxHeight, $maxHeight ?: null);
					}
					if ($transformRequired)
					{
						$image->transformByExif($orientation);
					}
					if ($optimizeRequired)
					{
						$success = $image->optimizeImage($this->tempFile);
						if ($success)
						{
							$this->isOptimized = true;
							$this->setFileName(pathinfo($this->fileName, PATHINFO_FILENAME) . '.webp');
						}
					}

					if ($image->save($this->tempFile))
					{
						$this->imageWidth = $image->getWidth();
						$this->imageHeight = $image->getHeight();
						clearstatcache();
						$this->fileSize = filesize($this->tempFile);
					}
				}
				else
				{
					// treat as non-image
					$this->isImage = false;
				}
			}
		}

		return $this;
	}
}